# /etc/init.d/service-manager.inc
# 1.5.9.2 - Fix a small bug in service_check_exec
# 1999-11-04, Version 1.6.0 by AGX

# --------------------------------------------------------
# Error Codes
# --------------------------------------------------------
# 50 - Eseguibile non trovato
# 51 - Pre-requisito non avviato
# 52 - Programma di configurazione inesistente
# 53 - Impossibile configurare il servizio (disabilitato?)
# 60 - servizio gia' avviato
# 61 - servizio gia' fermato
# --------------------------------------------------------
# Query results
# --------------------------------------------------------
#	100	- started
#	200	- started but not running (Error)
#	101	- stopped
#	201	- stopped but running (Error)
#	102	- cannot query (Error)
#	202	- cannot query but it is running (Hacker warning!)

# Constants
SUBSYS="/var/lock/subsys"

# Requisiti: whereis, pidof, grep, cut

# ===============================================================
# service_check_exec
# ---------------------------------------------------------------
# Controlla se il file eseguibile, associato con il daemon,
# e' presente.
# 	(1) Basename dell'eseguibile
#	(2) subsys
# ===============================================================
service_check_exec () {
  # Se non viene passato l'eseguibile non effettua la ricerca
  [ "$1" = "" ] && return 0
  # Cerca l'eseguibile (bisogna usare grep per evitare problemi
  # con i nomi di file che contengono dei punti)
  #OLD:whereis  "$1" | grep -q "$1"
  whereis  "$1" | cut -f 2- -d ':' | grep -q "$1"
  #  1>/dev/null
  # Se trovato esce normalmente
  [ $? -eq 0 ]  && return 0
  # Non trovato
  [ $RELAXED -eq 0 ] && echoc "RED" "File '$1' non trovato !"
  [ -f "$SUBSYS/$2" ] && rm -f "$SUBSYS/$2"
  [ $RELAXED -ne 0 ] && exit 0
  exit 50
}



# ===============================================================
# service_check_prereq
# ---------------------------------------------------------------
# Alcuni servizi dipendono da altri, ad esempio nfs richiede il
# portmapper che a sua volta richiede il servizio 'network'
# Il servizio prerequisito deve essere stato gia' avviato
# 	(1) Nome del servizio pre-requisito
# 	(2) Nome del servizio
# ===============================================================
service_check_prereq() {
  # Ignora se vuoto
  [ "$1" = "" ] && return 0
  if [ ! -f "$SUBSYS/$1" ]; then
    [ $RELAXED -eq 0 ] && echoc RED "Il prerequisito '$1' di '$2' non e' stato avviato !"
    [ -f "$SUBSYS/$2" ] && rm -f "$SUBSYS/$2"
    [ $RELAXED -ne 0 ] && exit 0
    exit 51
  fi
  return 0
}



# ===============================================================
# service_configure
# ---------------------------------------------------------------
# Prova a lanciare il file di configurazione del servizio
# 	(1) Nome del file di configurazione
# 	(2) Nome completo del programma di setup
# ===============================================================
service_configure () {

  # Se non presente, prova a lanciare il programma di configurazione
  if [ ! "$2" = "" ]; then
    if [ ! -x "$2" ]; then
      echoc RED "Il programma di configurazione '$2' non e' stato trovato !"
      exit 52
    else
     [ -d `dirname "$2"` ] && cd `dirname "$2"`
     exec $2
     exit 52
    fi
  else
    # --- OLD ---
    # If there is not a configuration program then lock the service
    # echoc RED "Impossibile configurare il servizio '$2', non e' stato assegnato un programma di configurazione !"
    # service_reset  "$1"
    # sleep 3
    # exit 53
    # --- NEW ---
    service_enable   "$1"
  fi
}



# ===============================================================
# service_check_config
# ---------------------------------------------------------------
# Se esiste carica il file di configurazione altrimenti prova a
# lanciare il programma di configurazione
# 	(1) Nome del file di configurazione
# 	(2) Nome completo del programma di setup
# ===============================================================
service_check_config () {

  # Ignora senza un file di configurazione
  [ "$1" = "" ] && return 0

  # Carica il file di configurazione
  if [ -f "/etc/sysconfig/$PROFILE/$1" ]; then
    . "/etc/sysconfig/$PROFILE/$1"

    # Controlla se il servizio e' abilitato
    case $RUNME in
      NONE|NEVER|LOCKED)
        [ $RELAXED -eq 0 ] && echoc YELLOW "Questo servizio e' stato bloccato dall'amministratore di sistema."
	[ $RELAXED -ne 0 ] && exit 0
        exit 53
	;;
      # Se e' RELAXED ed anche MANUAL "NON DEVE" avviare il servizio
      MANUAL*)
        [ $RELAXED -eq 1 ] && exit 53
        ;;
    esac
    return 0

  else

    # Non esiste allora configura il servizio
    service_configure  "$1"  "$2"

  fi
}



# ===============================================================
# service_check_running
# ---------------------------------------------------------------
# Controlla se il flag e' presente e se non relaxed controlla
# anche il pid del processo per vedere se e' stato avviato
# manualmente.
# Esce se il servizio sta girando
# 	(1) Nome del subsystem
# 	(2) Nome dell'eseguibile
# ===============================================================
service_check_running () {
  if [ $RELAXED -eq 1 ]; then
    # Se c'e' il flag esce subito
    #[ -f "$SUBSYS/$1" ] && exit 60
    [ -f "$SUBSYS/$1" ] && exit 0
    # Controlla il pid
    pidof $2 >/dev/null 2>/dev/null
    #[ $? -eq 0 ] && exit 60
    [ $? -eq 0 ] && exit 0
    return 0
  else
    pidof $2 >/dev/null 2>/dev/null
    if [ $? -eq 0 ]; then
      echoc YELLOW "Il servizio '$1' e' gia' stato avviato !"
      exit 60
    fi
    return 0
  fi
}



# ===============================================================
# service_check_flag
# ---------------------------------------------------------------
# Controlla solo il flag per quei servizi che non hanno pid
# 	(1) Nome del subsystem
# ===============================================================
service_check_flag () {
  if [ -f "$SUBSYS/$1" ]; then
    [ $RELAXED -eq 0 ] && echoc YELLOW "Il servizio '$1' e' gia' stato avviato !"
    [ $RELAXED -eq 0 ] && exit 0
    exit 60
  fi
  #This cause problems with swap.sh: touch "$SUBSYS/$1"
  return 0
}



# ===============================================================
# service_check_not_running
# ---------------------------------------------------------------
# Esce se il servizio NON sta girando
# 	(1) Nome del subsys
# 	(2) Nome dell'eseguibile
# ===============================================================
service_check_not_running () {
  if [ $RELAXED -eq 1 ]; then
    # Se c'e' il flag esce subito
    #[ ! -f "$SUBSYS/$1" ] && exit 61
    [ ! -f "$SUBSYS/$1" ] && exit 0
    # Controlla il pid
    pidof $2 >/dev/null 2>/dev/null
    #[ $? -ne 0 ] && exit 61
    [ $? -ne 0 ] && exit 0
    return 0
  else
    pidof $2 >/dev/null 2>/dev/null
    if [ $? -ne 0 ]; then
      echoc YELLOW "Il servizio '$1' e' gia' stato fermato !"
      [ -f "$SUBSYS/$1" ] && rm -f "$SUBSYS/$1"
      exit 61
    fi
    return 0
  fi
}


# ===============================================================
# service_check_not_flag
# ---------------------------------------------------------------
# Esce se il servizio NON e' stato avviato
# 	(1) Nome del subsys
# ===============================================================
service_check_not_flag () {
  if [ ! -f "$SUBSYS/$1" ]; then
    [ $RELAXED -eq 0 ] && echoc YELLOW "Il servizio '$1' e' gia' stato fermato !"
    [ $RELAXED -ne 0 ] && exit 0
    exit 60
  fi
  return 0
}



# ===============================================================
# service_check_profile
# ---------------------------------------------------------------
# Se RELAXED (=siamo al boot) ed il profilo e' stato cambiato
# Allora prima di avviare il servizio va inizializzato
# ===============================================================
service_check_profile () {
  [ $RELAXED -eq 0 ]  &&  return 0
  if [ "$PROFILE" != "`cat /etc/agx_lastprofile 2>/dev/null`" ]; then
    # Inizializza
    #echoc PURPLE "DEBUG[$0]: Il profilo e' cambiato, inizializzo questo servizio :-)"
    echoc PURPLE "Il profilo e' cambiato, inizializzo $0"
    initialize
  fi
}



# ===============================================================
# service_start
# ---------------------------------------------------------------
# Avvia l'eseguibile specificato
# 	(1) Command Line
# ===============================================================
service_start() {

	# echo basename of the program.
	BASENAME="`basename $1`"
	echo -n " ${BASENAME} "

	# And start it up.
	$* &
	REPLY=$?
	PID=$!

	#TODO controlla che il pid sia reale
	[ ! -f /var/run/${BASENAME}.pid ] && echo $PID >> /var/run/${BASENAME}.pid
        echo -n "($PID) "
	return $REPLY
}



# ===============================================================
# service_start_nodaemon
# ---------------------------------------------------------------
# Avvia l'eseguibile specificato
# 	(1) Command Line
# ===============================================================
service_start_nodaemon() {
	# echo basename of the program.
	BASENAME="`basename $1`"
	echo -n " ${BASENAME} "

	# And start it up.
	eval "$*"
	REPLY=$?
	PID=`pidof $1`

	#TODO controlla che il pid sia reale
	[ ! -f /var/run/${BASENAME}.pid ] && echo $PID >> /var/run/${BASENAME}.pid
        echo -n "($PID) "
	return $REPLY
}



# ===============================================================
# service_stop
# ---------------------------------------------------------------
# Ferma l'eseguibile specificato
# 	(1) Nome dell'eseguibile
# ===============================================================
service_stop() {

  BASENAME="`basename $1`"
  PID="`pidof $BASENAME`"

  # Kill it.
  if [ ! "$PID" = "" ] ; then
    echo -n " $BASENAME ($PID) "
    kill -TERM $PID
    # Assicura la chiusura del servizio
    PID="`pidof $BASENAME`"
    if [ ! "$PID" = "" ] ; then
      kill -KILL $PID
    fi
  else
    echoc YELLOW -n " Il processo '$BASENAME' non e' in esecuzione"
  fi

  # Remove pid file if any.
  rm -f /var/run/$BASENAME.pid
}



# ===============================================================
# service_stopall
# ---------------------------------------------------------------
# Ferma tutti i pid dell'eseguibile specificato
# 	(1) Nome dell'eseguibile
# ===============================================================
service_stopall() {

  BASENAME="`basename $1`"
  PID="`pidof $BASENAME`"

  # Kill it.
  if [ ! "$PID" = "" ] ; then
    echo -n " $BASENAME ($PID) "
    killall -TERM $PID
    # Assicura la chiusura del servizio
    PID="`pidof $BASENAME`"
    if [ ! "$PID" = "" ] ; then
      killall -KILL $PID
    fi
  else
    echoc YELLOW -n " Nessun processo di nome '$BASENAME' e' in esecuzione"
  fi

  # Remove pid file if any.
  rm -f /var/run/$BASENAME.pid
}



# ===============================================================
# service_sighup
# ---------------------------------------------------------------
# Manda il segnale HUP a tutti gli eseguibili
# 	(1) Nome dell'eseguibile
# ===============================================================
service_sighup() {

  BASENAME="`basename $1`"
  PID="`pidof $BASENAME`"

  # Kill it.
  if [ ! "$PID" = "" ] ; then
    echo -n " $BASENAME ($PID) "
    killall -HUP $BASENAME
  else
    echoc YELLOW -n " Il processo '$BASENAME' non e' in esecuzione !"
    # Remove pid file if any.
    rm -f /var/run/$BASENAME.pid
  fi
}



# ===============================================================
# service_query_mode
# ---------------------------------------------------------------
# Visualizza l'attuale sistema di avvio per il servizio
# 	(1) SERVICE_CONFIG
# ===============================================================
service_query_mode() {
  if [ -f "/etc/sysconfig/$PROFILE/$1" ]; then
    . /etc/sysconfig/$PROFILE/$1
    case $RUNME in
      MANUAL*)
        echo -n "(manual) "
        return 1
        ;;
      NONE|NEVER|LOCKED)
        echo -n "(locked) "
        return 2
        ;;
      *)
        echo -n "  (auto) "
        return 0
        ;;
    esac
  else
    echo -n "   (n/a) "
    return 3
  fi
}



# ===============================================================
# service_query
# ---------------------------------------------------------------
# This probe the daemon status
# reporting also if a daemon started was killed manually
# 	(1) SERVICE_SUBSYS
# 	(2) SERVICE_BIN
# 	(3) SERVICE_CONFIG
# ===============================================================
service_query () {
  # Display Daemon Starting Mode
  service_query_mode  "$3"

  # Check lock file
  if [ -f "$SUBSYS/$1" ]; then
    [ "$2" = "" ] && exit 100
    pidof $2 >/dev/null 2>/dev/null
    if [ $? = 0 ]; then
      exit 100
    else
      exit 200
    fi
  elif [ -f /etc/sysconfig/$PROFILE/$3 ]; then
    RCOD1=101
    RCOD2=201
    case $RUNME in
      NONE|NEVER|LOCKED)
        RCOD1=102
        RCOD2=202
      ;;
    esac
    [ "$2" = "" ] && exit $RCOD1
    pidof $2 >/dev/null 2>/dev/null
    if [ $? -ne 0 ]; then
      exit $RCOD1
    else
      exit $RCOD2
    fi

  else

    [ "$2" = "" ] && exit 102
    pidof $2 >/dev/null 2>/dev/null
    if [ $? -ne 0 ]; then
      exit 102
    else
      exit 202
    fi
  fi
}


# ===============================================================
# service_enable
# ---------------------------------------------------------------
# Abilita l'esecuzione del servizio a boot time
#	(1) SERVICE_CONFIG
# ===============================================================
service_enable() {
  if [ "$1" = "" ]; then
    echoc YELLOW "Attenzione il file di configurazione non e' stato specificato !"
    exit 52
  fi
  echoc GREEN "Il servizio '$SERVICE_NAME' e' ora abilitato all'avvio automatico in fase di 'boot'"
  echo "RUNME=AUTO" >> /etc/sysconfig/$PROFILE/$1
}



# ===============================================================
# service_reset
# ---------------------------------------------------------------
# Abilita l'esecuzione del servizio solo manualmente
#	(1) SERVICE_CONFIG
# ===============================================================
service_reset() {
  if [ "$1" = "" ]; then
    echoc YELLOW "Attenzione il file di configurazione non e' stato specificato !"
    exit 52
  fi
  echoc YELLOW "Il servizio '$SERVICE_NAME' puo' ora essere avviato solo manualmente usando 'service'"
  echo "RUNME=MANUAL" >> /etc/sysconfig/$PROFILE/$1
}



# ===============================================================
# service_disable
# ---------------------------------------------------------------
# Disabilita l'esecuzione del servizio
#	(1) SERVICE_CONFIG
# ===============================================================
service_disable() {
  if [ "$1" = "" ]; then
    echoc YELLOW "Attenzione il file di configurazione non e' stato specificato !"
    exit 52
  fi
  echoc YELLOW "Il servizio '$SERVICE_NAME' e' stato disabilitato fino a nuovo ordine."
  echo "RUNME=NEVER" >> /etc/sysconfig/$PROFILE/$1
}



# ===============================================================
# service_usage
# ---------------------------------------------------------------
# Visualizza l'help del comando service
# ===============================================================
service_usage () {
  echoc GREEN "AGX Service Manager 1.4 (???)"
  echo
  echoc YELLOW "Sintassi: service <nome_servizio> <comando> [relaxed]"
  echo
  echoc BLUE "Comandi:"
  echo " start        - avvia il servizio"
  echo " stop         - ferma il servizio"
  echo " restart      - ferma e poi riavvia il servizio"
  echo " reload       - se supportato, segnala al servizio di rileggere"
  echo "                i file di configurazione, altrimenti equivale a restart."
  echo "                Sinonimo di questo comando e' 'force-reload' "
  echo " query        - ritorna informazioni sullo stato del servizio"
  echo "                viene visualizzata prima la modalita' di avvio"
  echo "                e successivamente lo stato dello stesso."
  echo "                Sinonimi sono 'status' e 'probe' "
  echo " enable       - imposta il servizio in modo da essere avviato al boot"
  echo " disable      - blocca il servizio, non potra' essere avviato usando"
  echo "                il comando service. Sinonimo e' 'lock' "
  echo " reset        - blocca l'avvio in fase di boot del servizio che puo'"
  echo "                comunque essere avviato manualment mediante il comando"
  echo "                service"
  echo " configure    - richiama il file programma di configurazione del servizio"
  echo " init         - crea i filed ed i device necessari al servizio, se usato"
  echo "                il servizio va prima fermato"
  echo
}

# -----------------------------------------------------------------
# -----------------------------------------------------------------
# -----------------------------------------------------------------


